/**
* @license Apache-2.0
*
* Copyright (c) 2021 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var tape = require( 'tape' );
var proxyquire = require( 'proxyquire' );
var main = require( './../lib/main.js' );
var polyfill = require( './../lib/polyfill.js' );
var serialize = require( './../lib' );


// TESTS //

tape( 'main export is a function', function test( t ) {
	t.ok( true, __filename );
	t.strictEqual( typeof serialize, 'function', 'main export is a function' );
	t.end();
});

tape( 'if an environment support BigInts, the function includes BigInt operations', function test( t ) {
	var serialize = proxyquire( './../lib', {
		'@stdlib/assert/has-bigint-support': hasSupport
	});

	t.strictEqual( serialize, main, 'is expected value' );
	t.end();

	function hasSupport() {
		return true;
	}
});

tape( 'if an environment does not support BigInts, the function is a polyfill', function test( t ) {
	var serialize = proxyquire( './../lib', {
		'@stdlib/assert/has-bigint-support': hasSupport
	});

	t.strictEqual( serialize, polyfill, 'is expected value' );
	t.end();

	function hasSupport() {
		return false;
	}
});
