/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

var indexOf = require( '@stdlib/utils/index-of' );
var dtypes = require( './../lib' );

var DTYPES = dtypes();
var bool;

function isdtype( str ) {
	if ( indexOf( DTYPES, str ) === -1 ) {
		return false;
	}
	return true;
}

bool = isdtype( 'float64' );
console.log( bool );
// => true

bool = isdtype( 'int8' );
console.log( bool );
// => true

bool = isdtype( 'uint16' );
console.log( bool );
// => true

bool = isdtype( 'beep' );
console.log( bool );
// => false
