/**
* @license Apache-2.0
*
* Copyright (c) 2021 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#ifndef STDLIB_NDARRAY_BASE_UNARY_MACROS_H
#define STDLIB_NDARRAY_BASE_UNARY_MACROS_H

#include "stdlib/ndarray/base/unary/macros/constants.h"
#include "stdlib/ndarray/base/unary/macros/1d.h"
#include "stdlib/ndarray/base/unary/macros/2d.h"
#include "stdlib/ndarray/base/unary/macros/2d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/3d.h"
#include "stdlib/ndarray/base/unary/macros/3d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/4d.h"
#include "stdlib/ndarray/base/unary/macros/4d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/5d.h"
#include "stdlib/ndarray/base/unary/macros/5d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/6d.h"
#include "stdlib/ndarray/base/unary/macros/6d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/7d.h"
#include "stdlib/ndarray/base/unary/macros/7d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/8d.h"
#include "stdlib/ndarray/base/unary/macros/8d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/9d.h"
#include "stdlib/ndarray/base/unary/macros/9d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/10d.h"
#include "stdlib/ndarray/base/unary/macros/10d_blocked.h"
#include "stdlib/ndarray/base/unary/macros/nd.h"

#endif // !STDLIB_NDARRAY_BASE_UNARY_MACROS_H
