/**
* @license Apache-2.0
*
* Copyright (c) 2021 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var bench = require( '@stdlib/bench' );
var dtypes = require( './../../../dtypes' );
var bufferCtors = require( './../../../base/buffer-ctors' );
var isNumber = require( '@stdlib/assert/is-number' ).isPrimitive;
var isFunction = require( '@stdlib/assert/is-function' );
var pkg = require( './../package.json' ).name;
var dtypeEnum = require( './../lib' );


// VARIABLES //

var DTYPES = dtypes();


// MAIN //

bench( pkg, function benchmark( b ) {
	var buffers;
	var ctor;
	var dt;
	var i;

	buffers = [];
	for ( i = 0; i < DTYPES.length; i++ ) {
		ctor = bufferCtors( DTYPES[ i ] );
		if ( DTYPES[ i ] === 'binary' && isFunction( ctor.alloc ) ) {
			buffers.push( ctor.alloc( 10 ) );
		} else {
			buffers.push( new ctor( 10 ) );
		}
	}
	b.tic();
	for ( i = 0; i < b.iterations; i++ ) {
		dt = dtypeEnum( buffers[ i%buffers.length ] );
		if ( typeof dt !== 'number' ) {
			b.fail( 'should return a number' );
		}
	}
	b.toc();
	if ( !isNumber( dt ) ) {
		b.fail( 'should return a number' );
	}
	b.pass( 'benchmark finished' );
	b.end();
});
