/*
* @license Apache-2.0
*
* Copyright (c) 2021 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

// TypeScript Version: 2.0

/**
* Returns a list of ndarray data types to which a provided ndarray data type can be safely cast.
*
* ## Notes
*
* -   If not provided an ndarray data type, the function returns a casting table.
* -   If provided an unrecognized ndarray data type, the function returns `null`.
*
* @param dtype - ndarray data type
* @returns list of ndarray data types or null
*
* @example
* var list = safeCasts( 'float32' );
* // returns [...]
*/
declare function safeCasts( dtype?: string ): any;


// EXPORTS //

export = safeCasts;
